"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.start = exports.init = exports.loadLocale = exports.fetchLocale = void 0;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _client = require("@forrestjs/feature-network/client");

var _client2 = require("@forrestjs/feature-storage/client");

var _locale = _interopRequireDefault(require("./locale.query"));

var _locale2 = require("./locale.reducer");

var isCacheValid = function isCacheValid(ctime) {
  return function (_, getState) {
    var _getState = getState(),
        locale = _getState.locale;

    var cacheDuration = locale.cacheDuration;
    return new Date() - new Date(ctime) < cacheDuration;
  };
};

var localeExists = function localeExists(desiredLocale) {
  return function (dispatch, getState) {
    var _getState2 = getState(),
        locale = _getState2.locale;

    var current = locale.locales[desiredLocale];

    if (!current) {
      return false;
    }

    return dispatch(isCacheValid(current.ctime));
  };
};

var getCurrentLocale = function getCurrentLocale() {
  return function (dispatch, getState) {
    var _getState3 = getState(),
        locale = _getState3.locale,
        ssr = _getState3.ssr; // get locale from the current request using `express-locale`
    // during SSR, use the cookie in the client


    if (ssr.isServer()) {
      var req = ssr.getRequestHandler();
      return req.locale.language + "_" + req.locale.region;
    } else {
      return dispatch(_client2.cookie.get('locale', locale.locale));
    }
  };
};

var setCurrentLocale = function setCurrentLocale(locale) {
  return function (dispatch) {
    dispatch((0, _locale2.setLocale)(locale));
    dispatch(_client2.cookie.set('locale', locale));
  };
};

var addLocaleData = function addLocaleData(record) {
  return function (dispatch, getState) {
    var _getState4 = getState(),
        locale = _getState4.locale;

    var ctime = record.ctime || new Date();
    dispatch((0, _locale2.addLocale)(record.locale, record.messages, ctime));

    if (locale.cacheLocal === true) {
      dispatch(_client2.localStorage.setItem("locale::cache::" + record.locale, (0, _extends2["default"])({}, record, {
        ctime: ctime
      })));
    }
  };
};

var fetchLocale = function fetchLocale(locale) {
  return (
    /*#__PURE__*/
    function () {
      var _ref = (0, _asyncToGenerator2["default"])(
      /*#__PURE__*/
      _regenerator["default"].mark(function _callee(dispatch) {
        var res;
        return _regenerator["default"].wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return dispatch((0, _client.runQuery)(_locale["default"], {
                  locale: locale
                }));

              case 2:
                res = _context.sent;
                return _context.abrupt("return", res.data.locale);

              case 4:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      }));

      return function (_x) {
        return _ref.apply(this, arguments);
      };
    }()
  );
};

exports.fetchLocale = fetchLocale;

var loadLocale = function loadLocale(desiredLocale) {
  return (
    /*#__PURE__*/
    function () {
      var _ref2 = (0, _asyncToGenerator2["default"])(
      /*#__PURE__*/
      _regenerator["default"].mark(function _callee2(dispatch, getState) {
        var cachedData, remoteData;
        return _regenerator["default"].wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                if (!dispatch(localeExists(desiredLocale))) {
                  _context2.next = 3;
                  break;
                }

                dispatch(setCurrentLocale(desiredLocale));
                return _context2.abrupt("return");

              case 3:
                cachedData = null;
                remoteData = null; // try load from local storaged cache

                try {
                  cachedData = dispatch(_client2.localStorage.getItem("locale::cache::" + desiredLocale));

                  if (cachedData && dispatch(isCacheValid(cachedData.ctime))) {
                    remoteData = cachedData;
                  }
                } catch (err) {} // eslint-disable-line
                // fetch from the server


                if (remoteData) {
                  _context2.next = 15;
                  break;
                }

                _context2.prev = 7;
                _context2.next = 10;
                return dispatch(fetchLocale(desiredLocale));

              case 10:
                remoteData = _context2.sent;
                _context2.next = 15;
                break;

              case 13:
                _context2.prev = 13;
                _context2.t0 = _context2["catch"](7);

              case 15:
                if (remoteData) {
                  _context2.next = 23;
                  break;
                }

                console.log("[locale] failed to load locale: " + desiredLocale);

                if (!cachedData) {
                  _context2.next = 22;
                  break;
                }

                console.log('[locale] defaulting to an old cached definition');
                remoteData = cachedData;
                _context2.next = 23;
                break;

              case 22:
                return _context2.abrupt("return");

              case 23:
                dispatch(addLocaleData(remoteData));
                dispatch(setCurrentLocale(desiredLocale));

              case 25:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, null, [[7, 13]]);
      }));

      return function (_x2, _x3) {
        return _ref2.apply(this, arguments);
      };
    }()
  );
};

exports.loadLocale = loadLocale;

var init = function init() {
  return function (dispatch) {
    try {
      window.loadLocale = function (locale) {
        return dispatch(loadLocale(locale));
      };
    } catch (err) {} // eslint-disable-line

  };
};

exports.init = init;

var start = function start() {
  return function (dispatch, getState) {
    var _getState5 = getState(),
        ssr = _getState5.ssr;

    var current = dispatch(getCurrentLocale());
    return ssr["await"](dispatch(loadLocale(current)));
  };
};

exports.start = start;